using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.IO;
using System.Net.Mail;
using System.Text;
using System.Web;
using System.Web.Caching;
using System.Web.Hosting;
using BlogStarterKit.Utilities;

namespace BlogStarterKit
{
    public class BlogEngine
    {
        private SqlDatabase db = new SqlDatabase("Blog");

        // Categories

        public IEnumerable<Category> GetAllCategories()
        {
            return db.SelectAll<Category>("Categories");
        }

        public void SaveCategory(Category category)
        {
            if (category.CategoryID == -1)
            {
                db.Insert(category, "Categories");
            }
            else
            {
                db.Update(category, "Categories");
            }
        }

        public void DeleteCategory(Category category)
        {
            db.Delete(category, "Categories");
        }

        // Posts

        private IEnumerable<Post> GetPosts(int count, params object[] fieldValues)
        {
            return db.Select<Post>("Posts",
                "*, (SELECT Name FROM Categories WHERE Categories.CategoryID = Posts.CategoryID) AS CategoryName",
                count,
                "DatePosted DESC",
                fieldValues);
        }

        public IEnumerable<Post> GetAllPosts()
        {
            return GetPosts(Int32.MaxValue);
        }

        public IEnumerable<Post> GetRecentPosts(int count)
        {
            return GetPosts(count,
                "Draft", false);
        }

        public IEnumerable<Post> GetPostsByCategory(int categoryID, int count)
        {
            return GetPosts(count,
                    "Draft", false,
                    "CategoryID", categoryID);
        }

        public IEnumerable<Post> GetPostsByMonth(DateTime monthPosted)
        {
            return GetPosts(Int32.MaxValue,
                    "Draft", false,
                    "MonthPosted", monthPosted);
        }

        public IEnumerable<Post> GetPostByID(long postID, bool publishedOnly)
        {
            if (publishedOnly)
            {
                return GetPosts(1,
                        "PostID", postID,
                        "Draft", false);
            }
            else
            {
                return GetPosts(1,
                        "PostID", postID);
            }
        }

        public IEnumerable<Post> GetPostByUrl(string permalinkUrl)
        {
            return GetPosts(1,
                    "PermalinkUrl", permalinkUrl,
                    "Draft", false);
        }

        public void SavePost(Post post)
        {
            if (post.PostID == -1)
            {
                db.Insert(post, "Posts");
            }
            else
            {
                db.Update(post, "Posts");
            }
        }

        public void DeletePost(Post post)
        {
            db.Delete(post, "Posts");
        }

        public void PublishAllPosts()
        {
            db.ExecuteNonQuery("UPDATE Posts SET Draft = FALSE WHERE Draft = TRUE");
        }

        // Feedback

        public IEnumerable<Feedback> GetApprovedFeedbackByPost(long postID)
        {
            return db.Select<Feedback>("Feedback", "*", Int32.MaxValue, "DateCreated DESC",
                "PostID", postID,
                "Approved", true);
        }

        public IEnumerable<Feedback> GetPendingFeedback()
        {
            return db.Select<Feedback>("Feedback", "*", Int32.MaxValue, null,
                "Approved", false);
        }

        public void ApproveFeedback(long feedbackID)
        {
            db.ExecuteNonQuery("UPDATE Feedback SET Approved = TRUE WHERE FeedbackID = :FeedbackID",
                ":FeedbackID", feedbackID);
        }

        public void SaveFeedback(Feedback feedback)
        {
            if (feedback.FeedbackID == -1)
            {
                bool isAuthenticated = HttpContext.Current.User.Identity.IsAuthenticated;

                feedback.DateCreated = DateTime.Now;
                if (isAuthenticated)
                {
                    feedback.Approved = true;
                }

                db.Insert(feedback, "Feedback");

                if (!isAuthenticated)
                {
                    if (ConfigurationManager.AppSettings["sendEmailOnNewFeedback"] == "true")
                    {
                        string postTitle = db.ExecuteScalar<string>(
                            "SELECT Title FROM Posts WHERE PostID = :PostID",
                            ":PostID", feedback.PostID);
                        SendFeedbackEmail(postTitle, feedback);
                    }
                }
            }
            else
            {
                db.Update(feedback, "Feedback");
            }
        }

        public void DeleteFeedback(Feedback feedback)
        {
            db.Delete(feedback, "Feedback");
        }

        // Statistics

        public DateTime GetLastUpdateTime()
        {
            // TODO!
            return DateTime.Now;
        }

        public IEnumerable<PostMonth> GetPostCountByMonth(int months)
        {
            return db.SelectSql<PostMonth>(
                @"SELECT MonthPosted AS Month, CAST(COUNT(1) AS int) AS PostCount
                    FROM Posts
                    GROUP BY MonthPosted
                    ORDER BY MonthPosted DESC LIMIT :Count",
                ":Count", months);
        }

        // Helpers

        public static void SendFeedbackEmail(string postTitle, Feedback feedback)
        {
            string emailTemplate;

            emailTemplate = (string)HostingEnvironment.Cache["feedbackEmail"];
            if (emailTemplate == null)
            {
                string filename = HostingEnvironment.MapPath(ConfigurationManager.AppSettings["emailTemplate"]);
                StreamReader reader = new StreamReader(filename);
                using (reader)
                {
                    emailTemplate = reader.ReadToEnd();
                }
                HostingEnvironment.Cache.Add("feedbackEmail", emailTemplate,
                    new CacheDependency(filename),
                    Cache.NoAbsoluteExpiration, Cache.NoSlidingExpiration,
                    CacheItemPriority.Normal, null);
            }

            Uri uri = HttpContext.Current.Request.Url;
            MailMessage message = new MailMessage();
            message.To.Add(new MailAddress(ConfigurationManager.AppSettings["ownerEmail"]));
            message.Subject = ConfigurationManager.AppSettings["emailSubject"];
            message.Body = String.Format(emailTemplate,
                                uri.GetLeftPart(UriPartial.Authority) + HttpContext.Current.Request.ApplicationPath,
                                feedback.PostID, postTitle,
                                feedback.Email, feedback.Contents, 
                                feedback.FeedbackID);
            message.IsBodyHtml = true;

            SmtpClient client = new SmtpClient();
            client.Send(message);
        }
    }
}
