using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Security.Cryptography;
using System.Web;
using System.Web.Security;

namespace BlogStarterKit.Utilities
{
    public class Captcha
    {
        private const int _imageWidth = 140;
        private const int _imageHeight = 50;
        private const int _challengeKeyLength = 6;
        private const string _characterMap = "ABCDEGHJKMNRSTUWXY235689";
        private static List<Captcha> _captchas = new List<Captcha>();
        private const int _minimumTimeout = 900;
        private const int _maximumTimeout = 1200;
        private const int _captchaCacheSize = 5;

        private string _id;
        private DateTime _goodUntil;
        private string _challengeKey;
        private Byte[] _imageBytes;
        private Byte[] _audioBytes;
        private int _useCount;

        private Captcha()
        {
            _goodUntil = DateTime.Now.AddSeconds(_maximumTimeout);
            _id = Guid.NewGuid().ToString();
            _challengeKey = GenerateKey();
            _imageBytes = GenerateImage();
            _useCount = 0;
        }

        private string GenerateKey()
        {
            RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();
            Byte[] bytes = new Byte[_challengeKeyLength];
            rng.GetBytes(bytes);
            char[] chars = new char[bytes.Length];
            for (int i = 0; i < bytes.Length; i++)
            {
                chars[i] = _characterMap[bytes[i] % _characterMap.Length];
            }
            return new String(chars);
        }

        private Byte[] GenerateImage()
        {
            RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();
            Byte[] rand = new Byte[200];
            rng.GetBytes(rand);
            int i = 0;

            Bitmap bmp = new Bitmap(_imageWidth, _imageHeight, PixelFormat.Format24bppRgb);
            Bitmap cloneBmp = null;
            Graphics g = null;
            LinearGradientBrush backgroundBrush = null;
            LinearGradientBrush textBrush = null;
            SolidBrush[] circleBrush = new SolidBrush[3];
            Font font = null;
            GraphicsPath path = null;

            try
            {
                g = Graphics.FromImage(bmp);
                g.SmoothingMode = SmoothingMode.AntiAlias;
                Rectangle r = new Rectangle(0, 0, _imageWidth, _imageHeight);
                backgroundBrush = new LinearGradientBrush(
                        new RectangleF(0, 0, _imageWidth, _imageHeight),
                        Color.FromArgb(rand[i++] / 2 + 128, rand[i++] / 2 + 128, 255),
                        Color.FromArgb(255, rand[i++] / 2 + 128, rand[i++] / 2 + 128),
                        rand[i++] * 360 / 256);
                g.FillRectangle(backgroundBrush, r);

                for (int br = 0; br < circleBrush.Length; br++)
                {
                    circleBrush[br] = new SolidBrush(Color.FromArgb(128, rand[i++], rand[i++], rand[i++]));
                }

                for (int circle = 0; circle < 30; circle++)
                {
                    int radius = rand[i++] % 10;
                    g.FillEllipse(circleBrush[circle % 2],
                        rand[i++] * _imageWidth / 256,
                        rand[i++] * _imageHeight / 256,
                        radius, radius);
                }

                font = new Font("Times New Roman", _imageHeight / 2, FontStyle.Bold);
                StringFormat format = new StringFormat();
                format.Alignment = StringAlignment.Center;
                format.LineAlignment = StringAlignment.Center;

                path = new GraphicsPath();
                path.AddString(_challengeKey, font.FontFamily,
                    (int)font.Style, font.Size, r, format);

                textBrush = new LinearGradientBrush(
                        new RectangleF(0, 0, _imageWidth, _imageHeight),
                        Color.FromArgb(rand[i] % 128, rand[i] % 128, rand[i++] % 128),
                        Color.FromArgb(rand[i] % 128, rand[i] % 128, rand[i++] % 128),
                        rand[i++] * 360 / 256);
                g.FillPath(textBrush, path);

                cloneBmp = (Bitmap)bmp.Clone();

                int distortionSeed = rand[i++];
                double distortion = distortionSeed > 128 ? 5 + (distortionSeed - 128) % 5 : -5 - distortionSeed % 5;
                for (int y = 0; y < _imageHeight; y++)
                {
                    for (int x = 0; x < _imageWidth; x++)
                    {
                        // Adds a simple wave
                        int newX = (int)(x + (distortion * Math.Sin(Math.PI * y / 96.0)));
                        int newY = (int)(y + (distortion * Math.Cos(Math.PI * x / 64.0)));
                        if (newX < 0 || newX >= _imageWidth)
                        {
                            newX = 0;
                        }
                        if (newY < 0 || newY >= _imageHeight)
                        {
                            newY = 0;
                        }
                        bmp.SetPixel(x, y, cloneBmp.GetPixel(newX, newY));
                    }
                }

                MemoryStream stream = new MemoryStream();
                bmp.Save(stream, ImageFormat.Jpeg);
                return stream.ToArray();
            } finally
            {
                if (backgroundBrush != null)
                {
                    backgroundBrush.Dispose();
                }
                if (textBrush != null)
                {
                    textBrush.Dispose();
                }
                for (int br = 0; br < circleBrush.Length; br++)
                {
                    if (circleBrush[br] != null)
                    {
                        circleBrush[br].Dispose();
                    }
                }
                if (font != null)
                {
                    font.Dispose();
                }
                if (path != null)
                {
                    path.Dispose();
                }
                if (g != null)
                {
                    g.Dispose();
                }
                if (bmp != null)
                {
                    bmp.Dispose();
                }
                if (cloneBmp != null)
                {
                    cloneBmp.Dispose();
                }
            }
        }

        public Byte[] GetAudioBytes(string soundFolder)
        {
            // TODO: Get audio.
            return null;
        }

        public static string GetCaptcha()
        {
            Captcha selectedCaptcha = null;
            DateTime now = DateTime.Now;

            lock (_captchas)
            {
                if (_captchas.Count >= _captchaCacheSize)
                {
                    int minUseCount = Int32.MaxValue;
                    for (int i = 0; i < _captchas.Count; i++)
                    {
                        Captcha captcha = _captchas[i];
                        if (now.AddSeconds(_minimumTimeout) < captcha._goodUntil &&
                            captcha._useCount < minUseCount)
                        {
                            selectedCaptcha = captcha;
                            minUseCount = captcha._useCount;
                        }
                        else if (now > captcha._goodUntil)
                        {
                            _captchas.RemoveAt(i);
                            i--;
                        }
                    }
                }

                if (selectedCaptcha == null)
                {
                    selectedCaptcha = new Captcha();
                    _captchas.Add(selectedCaptcha);
                }

                selectedCaptcha._useCount++;
                return selectedCaptcha._id;
            }
        }

        public static bool ValidateCaptcha(string id, string response)
        {
            foreach (Captcha captcha in _captchas)
            {
                if (captcha != null && captcha._id == id)
                {
                    return captcha._challengeKey == response;
                }
            }
            return false;
        }

        public static Byte[] GetCaptchaImage(string id)
        {
            foreach (Captcha captcha in _captchas)
            {
                if (captcha != null && captcha._id == id)
                {
                    return captcha._imageBytes;
                }
            }
            return null;
        }

    }
}