using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace BlogStarterKit.Utilities
{

    public abstract class ContentFormatter
    {
        public abstract string FormatToHtml(string content);
        public abstract string FormatFromHtml(string htmlContent);
    }

    public class FeedbackFormatter : ContentFormatter
    {
        private static Regex _urlRegex = new Regex(@"(?<url>(http|https|ftp)://\S*\w)(?<rest>\s|\W)", RegexOptions.Compiled);
        private static Regex _codeRegex = new Regex(@"\[code\](?<code>.*)\[/code\]", RegexOptions.Compiled);
        private static Regex _anchorRegex = new Regex(@"<a .* href=""(?<url>(^"")*)"".*</a>", RegexOptions.Compiled);

        public override string FormatFromHtml(string htmlContent)
        {
            StringBuilder sb = new StringBuilder();
            StringReader reader = new StringReader(htmlContent);
            string line;
            
            while ((line = reader.ReadLine()) != null)
            {
                if (line == "<p>" || line == "<br />")
                {
                    // Ignore.
                }
                else if (line == "</p>")
                {
                    sb.AppendLine();
                }
                else if (line == "<div class=\"code\"><pre>") 
                {
                    sb.AppendLine("[code]");
                }
                else if (line == "</pre></div>")
                {
                    sb.AppendLine("[/code]");
                }
                else
                {
                    line = _anchorRegex.Replace(line, @"{url}");
                    sb.AppendLine(line);
                }
            }

            return sb.ToString();
        }

        public override string FormatToHtml(string content)
        {
            StringBuilder sb = new StringBuilder();
            StringReader reader = new StringReader(content);
            bool inParagraph = false;
            bool inCode = false;
            string line = null;

            while ((line = reader.ReadLine()) != null)
            {
                // Convert paragraph markers to paragraphs.

                string trimmedLine = line.Trim();

                if (trimmedLine.Length == 0 && !inCode)
                {
                    if (inParagraph)
                    {
                        sb.AppendLine("</p>");
                    }
                    inParagraph = false;
                }
                else if (String.Compare(trimmedLine, "[code]", true) == 0 && !inCode)
                {
                    if (inParagraph)
                    {
                        sb.AppendLine("</p>");
                    }
                    inParagraph = false;
                    sb.AppendLine("<div class=\"code\"><pre>");
                    inCode = true;
                }
                else if (String.Compare(trimmedLine, "[/code]", true) == 0 && inCode)
                {
                    sb.AppendLine("</pre></div>");
                    inCode = false;
                    inParagraph = false;
                }
                else
                {
                    // Encode HTML tags.
                    line = line.Replace("<", "&lt;");
                    line = line.Replace(">", "&gt;");

                    // Convert URLs to hyperlinks.
                    line = _urlRegex.Replace(line, @"<a target=""_new"" href=""${url}"">${url}</a>${rest}");

                    if (inCode)
                    {
                        sb.AppendLine(line);
                    }
                    else
                    {
                        if (!inParagraph)
                        {
                            sb.AppendLine("<p>");
                            inParagraph = true;
                        }
                        else
                        {
                            sb.AppendLine("<br/>");
                        }
                        sb.AppendLine(line);
                    }
                }
            }

            if (inCode)
            {
                sb.AppendLine("</pre></div>");
            }
            else if (inParagraph)
            {
                sb.AppendLine("</p>");
            }

            return sb.ToString();
        }
    }
}