using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using Npgsql;
using NpgsqlTypes;
using System.Reflection;
using System.Text;

namespace BlogStarterKit.Utilities
{
    public class SqlDatabase
    {
        private string _connectionString;

        public SqlDatabase(string connectionStringName)
        {
            _connectionString = ConfigurationManager.ConnectionStrings[connectionStringName].ConnectionString;
        }

        public IEnumerable<T> SelectSql<T>(string query, params object[] parameters) where T : new()
        {
            return (IEnumerable<T>)SelectSql(new Factory<T>(), query, parameters);
        }

        public IEnumerable SelectSql(Type datatype, string query, params object[] parameters)
        {
            return SelectSql(new DynamicFactory(datatype), query, parameters);
        }

        private IEnumerable SelectSql(IFactory factory, string query, object[] parameters)
        {
            NpgsqlConnection connection = CreateConnection();
            NpgsqlCommand command = CreateCommand(connection, query, parameters);
            NpgsqlDataReader reader = command.ExecuteReader(CommandBehavior.CloseConnection);

            Type datatype = factory.GetDataType();
            IList items = factory.ConstructList();
            PropertyInfo[] fieldProperties = null;
	    
            while (reader.Read())
            {
                if (fieldProperties == null)
                {
                    fieldProperties = new PropertyInfo[reader.FieldCount];
                    for (int i = 0; i < fieldProperties.Length; i++)
                    {
                        fieldProperties[i] = datatype.GetProperty(reader.GetName(i), BindingFlags.Public | BindingFlags.Instance | BindingFlags.IgnoreCase);
                    }
                }

                object item = factory.ConstructElement();
                for (int i = 0; i < fieldProperties.Length; i++)
                {
                    object fieldValue = reader.GetValue(i);
                    if (fieldValue != DBNull.Value)
                    {
                        fieldProperties[i].SetValue(item, fieldValue, null);
                    }
                }

                items.Add(item);
            }

            return items;
        }

        public IEnumerable<T> SelectAll<T>(string tableName) where T : new()
        {
            return Select<T>(tableName, "", Int32.MaxValue, "");
        }

        public IEnumerable<T> Select<T>(string tableName, string fields, int count, string orderBy, params object[] fieldValues) where T : new()
        {
            return (IEnumerable<T>)Select(new Factory<T>(), tableName, fields, count, orderBy, fieldValues);
        }

        public IEnumerable Select(Type datatype, string tableName, string fields, int count, string orderBy, params object[] fieldValues)
        {
            return Select(new DynamicFactory(datatype), tableName, fields, count, orderBy, fieldValues);
        }

        private IEnumerable Select(IFactory factory, string tableName, string fields, int count, string orderBy, params object[] fieldValues)
        {
            List<object> parameters = new List<object>();
            StringBuilder sb = new StringBuilder();
            sb.Append("SELECT ");

            sb.Append(String.IsNullOrEmpty(fields) ? "*" : fields);

            sb.Append(" FROM ");
            sb.Append(tableName);

            if (fieldValues != null && fieldValues.Length > 0)
            {
                sb.Append(" WHERE ");
                for (int i = 0; i < fieldValues.Length; i += 2)
                {
                    if (i > 0)
                    {
                        sb.Append(" AND ");
                    }
                    sb.AppendFormat("({0} = :{0})", fieldValues[i]);
                    parameters.Add(":" + fieldValues[i]);
                    parameters.Add(fieldValues[i + 1]);
                }
            }

            if (!String.IsNullOrEmpty(orderBy))
            {
                sb.Append(" ORDER BY ");
                sb.Append(orderBy);
            }

	    if (count > 0 && count != Int32.MaxValue)
            {
                sb.Append(" LIMIT :Count");
                parameters.Add(":Count");
                parameters.Add(count);
            }
	    
            return SelectSql(factory, sb.ToString(), parameters.ToArray());
        }

        public void Update<T>(T item, string tableName)
        {
            Update((object)item, tableName);
        }

        public void Update(object item, string tableName)
        {
            PropertyInfo[] fields = GetFields(item.GetType(), tableName);

            StringBuilder sb = new StringBuilder();
            sb.Append("UPDATE ");
            sb.Append(tableName);
            sb.Append(" SET ");
            for (int i = 1; i < fields.Length; i++)
            {
                if (i > 1)
                {
                    sb.Append(',');
                }
                sb.Append(fields[i].Name);
                sb.Append("=:");
                sb.Append(fields[i].Name);
            }
            sb.Append(" WHERE ");
            sb.Append(fields[0].Name);
            sb.Append(" = :");
            sb.Append(fields[0].Name);

            Type datatype = item.GetType();
            object[] parameters = new object[fields.Length * 2];
            for (int i = 0; i < fields.Length; i++)
            {
                parameters[i * 2] = fields[i].Name;
                parameters[i * 2 + 1] = fields[i].GetValue(item, null);
            }

            ExecuteNonQuery(sb.ToString(), parameters);
        }

        public void Insert<T>(T item, string tableName)
        {
            Insert((object)item, tableName);
        }

        public void Insert(object item, string tableName)
        {
            PropertyInfo[] fields = GetFields(item.GetType(), tableName);

            StringBuilder sb = new StringBuilder();
            sb.Append("INSERT INTO ");
            sb.Append(tableName);
            sb.Append(" (");
            for (int i = 1; i < fields.Length; i++)
            {
                if (i > 1)
                {
                    sb.Append(',');
                }
                sb.Append(fields[i].Name);
            }
            sb.Append(") VALUES (");
            for (int i = 1; i < fields.Length; i++)
            {
                if (i > 1)
                {
                    sb.Append(',');
                }
                sb.Append(':');
                sb.Append(fields[i].Name);
            }
            sb.Append("); SELECT lastval()");

            Type datatype = item.GetType();

            object[] parameters = new object[fields.Length * 2 - 2];
            for (int i = 1; i < fields.Length; i++)
            {
                parameters[i * 2 - 2] = fields[i].Name;
                parameters[i * 2 - 1] = fields[i].GetValue(item, null);
            }

            object identity = ExecuteScalar(sb.ToString(), parameters);
            if (fields[0].PropertyType == typeof(int))
            {
                fields[0].SetValue(item, Convert.ToInt32(identity), null);
            }
            else if (fields[0].PropertyType == typeof(long))
            {
                fields[0].SetValue(item, Convert.ToInt64(identity), null);
            }
            else
            {
                fields[0].SetValue(item, identity, null);
            }
        }

        public void Delete<T>(T item, string tableName)
        {
            Delete((object)item, tableName);
        }

        public void Delete(object item, string tableName)
        {
            PropertyInfo[] fields = GetFields(item.GetType(), tableName);
            String command = String.Format("DELETE FROM {0} WHERE {1}=:{1}", tableName, fields[0].Name);
            ExecuteNonQuery(command, ":" + fields[0].Name, fields[0].GetValue(item, null));
        }

        public void ExecuteNonQuery(string commandText, params object[] parameters)
        {
            NpgsqlConnection conn = CreateConnection();
            NpgsqlCommand command = CreateCommand(conn, commandText, parameters);
            command.ExecuteNonQuery();
            conn.Close();
        }

        public T ExecuteScalar<T>(string commandText, params object[] parameters)
        {
            return (T)ExecuteScalar(commandText, parameters);
        }

        private object ExecuteScalar(string commandText, object[] parameters)
        {
            NpgsqlConnection conn = CreateConnection();
            NpgsqlCommand command = CreateCommand(conn, commandText, parameters);
            object returnValue = command.ExecuteScalar();
            conn.Close();
            return returnValue;
        }

        private NpgsqlConnection CreateConnection()
        {
            NpgsqlConnection conn = new NpgsqlConnection(_connectionString);
            conn.Open();
            return conn;
        }

        private NpgsqlCommand CreateCommand(NpgsqlConnection conn, string commandText, object[] parameters)
        {
            NpgsqlCommand command = new NpgsqlCommand(commandText, conn);

            if (parameters != null && parameters.Length > 0)
            {
                for (int i = 0; i < parameters.Length; i += 2)
                {
                    command.Parameters.Add(new NpgsqlParameter((string)parameters[i], parameters[i + 1]));
                }
            }

            return command;
        }

        private static Dictionary<Type, PropertyInfo[]> _tableFields = new Dictionary<Type, PropertyInfo[]>();

        private PropertyInfo[] GetFields(Type datatype, string tableName)
        {
            if (_tableFields.ContainsKey(datatype))
            {
                return _tableFields[datatype];
            }

            NpgsqlConnection conn = CreateConnection();
            NpgsqlCommand command = CreateCommand(conn, String.Format("SELECT * FROM {0}", tableName), null);
            NpgsqlDataReader reader = command.ExecuteReader(CommandBehavior.CloseConnection | CommandBehavior.SchemaOnly | CommandBehavior.KeyInfo);

            DataTable schema = reader.GetSchemaTable();
            int fieldCount = schema.Rows.Count;
            PropertyInfo[] fields = new PropertyInfo[fieldCount];
            int fieldIndex = 1;
            foreach (DataRow row in schema.Rows)
            {
                PropertyInfo field = datatype.GetProperty((string)row["ColumnName"], BindingFlags.Public | BindingFlags.Instance |
							  BindingFlags.IgnoreCase);
                if ((bool)row["IsKey"] && fields[0] == null)
                {
                    fields[0] = field;
                }
                else
                {
                    fields[fieldIndex++] = field;
                }
            }

            _tableFields.Add(datatype, fields);
            return fields;
        }

        private interface IFactory
        {
            Type GetDataType();
            IList ConstructList();
            object ConstructElement();
        }

        private class Factory<T> : IFactory where T : new()
        {
            Type IFactory.GetDataType()
            {
                return typeof(T);
            }

            IList IFactory.ConstructList()
            {
                return new List<T>();
            }

            object IFactory.ConstructElement()
            {
                return new T();
            }
        }

        private class DynamicFactory : IFactory
        {
            private Type _datatype;

            internal DynamicFactory(Type datatype)
            {
                _datatype = datatype;
            }

            Type IFactory.GetDataType()
            {
                return _datatype;
            }

            IList IFactory.ConstructList()
            {
                return new ArrayList();
            }

            object IFactory.ConstructElement()
            {
                return Activator.CreateInstance(_datatype);
            }
        }
    }
}