SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = true;
SET client_min_messages = warning;
SET escape_string_warning = off;
-- PostgreSQL membership/role requiremets

CREATE TABLE Users (
	pId										character(36)			NOT NULL,
	Username									character varying(255)	NOT NULL,
	ApplicationName							character varying(255)	NOT NULL,
	Email										character varying(128)	NULL,
	Comment									character varying(128)	NULL,
	Password									character varying(255)	NOT NULL,
	PasswordQuestion							character varying(255)	NULL,
	PasswordAnswer							character varying(255)	NULL,
	IsApproved								boolean					NULL, 
	LastActivityDate							timestamptz				NULL,
	LastLoginDate								timestamptz				NULL,
	LastPasswordChangedDate					timestamptz				NULL,
	CreationDate								timestamptz				NULL, 
	IsOnLine									boolean					NULL,
	IsLockedOut								boolean					NULL,
	LastLockedOutDate							timestamptz				NULL,
	FailedPasswordAttemptCount				integer					NULL,
	FailedPasswordAttemptWindowStart			timestamptz				NULL,
	FailedPasswordAnswerAttemptCount			integer					NULL,
	FailedPasswordAnswerAttemptWindowStart	timestamptz				NULL,
	CONSTRAINT users_pkey PRIMARY KEY (pId),
	CONSTRAINT users_username_application_unique UNIQUE (Username, ApplicationName)
);

CREATE INDEX users_email_index ON Users (Email);
CREATE INDEX users_islockedout_index ON Users (IsLockedOut);

-- PostgreSQL 8 Role Provider Schema

CREATE TABLE Roles (
	Rolename				character varying(255)	NOT NULL,
	ApplicationName		character varying(255)	NOT NULL,
	CONSTRAINT roles_pkey PRIMARY KEY (Rolename, ApplicationName)
);

CREATE TABLE UsersInRoles (
	Username				character varying(255)	NOT NULL,
	Rolename				character varying(255)	NOT NULL,
	ApplicationName		character varying(255)	NOT NULL,
	CONSTRAINT usersinroles_pkey PRIMARY KEY (Username, Rolename, ApplicationName),
	CONSTRAINT usersinroles_username_fkey FOREIGN KEY (Username, ApplicationName) REFERENCES Users (Username, ApplicationName) ON DELETE CASCADE,
	CONSTRAINT usersinroles_rolename_fkey FOREIGN KEY (Rolename, ApplicationName) REFERENCES Roles (Rolename, ApplicationName) ON DELETE CASCADE
);

-- PostgreSQL 8 Profile Provider Schema

CREATE TABLE Profiles (
	pId					character(36)			NOT NULL,
	Username				character varying(255)	NOT NULL,
	ApplicationName		character varying(255)	NOT NULL,
	IsAnonymous			boolean					NULL,
	LastActivityDate		timestamptz				NULL,
	LastUpdatedDate		timestamptz				NULL,
	CONSTRAINT profiles_pkey PRIMARY KEY (pId),
	CONSTRAINT profiles_username_application_unique UNIQUE (Username, ApplicationName),
	CONSTRAINT profiles_username_fkey FOREIGN KEY (Username, ApplicationName) REFERENCES Users (Username, ApplicationName) ON DELETE CASCADE
);

CREATE INDEX profiles_isanonymous_index ON Profiles (IsAnonymous);

CREATE TABLE ProfileData (
	pId					character(36)			NOT NULL,
	Profile				character(36)			NOT NULL,
	Name					character varying(255)	NOT NULL,
	ValueString			text					NULL,
	ValueBinary			bytea					NULL,
	CONSTRAINT profiledata_pkey PRIMARY KEY (pId),
	CONSTRAINT profiledata_profile_name_unique UNIQUE (Profile, Name),
	CONSTRAINT profiledata_profile_fkey FOREIGN KEY (Profile) REFERENCES Profiles (pId) ON DELETE CASCADE
);

-- PostgreSQL 8 Session-Store Provider Schema

CREATE TABLE Sessions (
	SessionId				character varying(80)	NOT NULL,
	ApplicationName		character varying(255)	NOT NULL,
	Created				timestamptz				NOT NULL,
	Expires				timestamptz				NOT NULL,
	Timeout				integer					NOT NULL,
	Locked				boolean					NOT NULL,
	LockId				integer					NOT NULL,
	LockDate				timestamptz				NOT NULL,
	Data					text					NULL,
	Flags					integer					NOT NULL,
	CONSTRAINT sessions_pkey PRIMARY KEY (SessionId, ApplicationName)
);
-- ported from upstream


CREATE TABLE Categories (
	CategoryID serial NOT NULL PRIMARY KEY,
	Name varchar(50) NOT NULL,
	Description varchar(500) NULL
);

CREATE INDEX IX_Categories_Name ON Categories (
	Name
);

CREATE TABLE Feedback (
	FeedbackID serial NOT NULL PRIMARY KEY,
	PostID bigint NOT NULL,
	Name varchar(100) NOT NULL,
	Email varchar(100) NULL,
	Blog varchar(200) NULL,
	Contents text NOT NULL,
	Approved boolean NOT NULL,
	DateCreated timestamptz NOT NULL
);

CREATE INDEX IX_Feedback ON Feedback (
	Approved
);

CREATE TABLE Posts (
	PostID serial NOT NULL PRIMARY KEY,
	Title varchar(200) NOT NULL,
	Subtitle varchar(200) NULL,
	CategoryID int NULL,
	MonthPosted timestamptz NOT NULL,
	DatePosted timestamptz NOT NULL DEFAULT (now()),
	Draft boolean NOT NULL,
	Contents text NOT NULL,
	PermalinkUrl varchar(200) NULL
);

CREATE INDEX IX_Posts_DatePosted ON Posts (
	DatePosted
);

CREATE INDEX IX_Posts_MonthPosted ON Posts (
	MonthPosted
);

CREATE INDEX IX_Posts_PermalinkUrl ON Posts (
	PermalinkUrl
);

ALTER TABLE Feedback ADD  CONSTRAINT FK_Feedback_Posts FOREIGN KEY(PostID)
REFERENCES Posts (PostID)
ON DELETE CASCADE;

ALTER TABLE Posts  ADD  CONSTRAINT FK_Posts_Categories FOREIGN KEY(CategoryID)
REFERENCES Categories (CategoryID)
ON DELETE SET NULL;

-- alter object owners
ALTER TABLE Users OWNER TO test;
ALTER INDEX users_email_index OWNER TO test;
ALTER INDEX users_islockedout_index OWNER TO test;
ALTER TABLE Roles OWNER TO test;
ALTER TABLE UsersInRoles OWNER TO test;
ALTER TABLE Profiles OWNER TO test;
ALTER INDEX profiles_isanonymous_index OWNER TO test;
ALTER TABLE ProfileData OWNER TO test;
ALTER TABLE Categories OWNER TO test;
ALTER INDEX IX_Categories_Name OWNER TO test;
ALTER TABLE Feedback OWNER TO test;
ALTER INDEX IX_Feedback OWNER TO test;
ALTER TABLE Posts OWNER TO test;
ALTER INDEX IX_Posts_DatePosted OWNER TO test;
ALTER INDEX IX_Posts_MonthPosted OWNER TO test;
ALTER INDEX IX_Posts_PermalinkUrl OWNER TO test;

-- test user data
INSERT INTO roles (rolename, applicationname) VALUES ('Administrators', '/');
INSERT INTO roles (rolename, applicationname) VALUES ('Guests', '/');

INSERT INTO users (pid, username, applicationname, email, "comment", "password", passwordquestion, passwordanswer, isapproved, lastactivitydate, lastlogindate, lastpasswordchangeddate, creationdate, isonline, islockedout, lastlockedoutdate, failedpasswordattemptcount, failedpasswordattemptwindowstart, failedpasswordanswerattemptcount, failedpasswordanswerattemptwindowstart) VALUES ('8bd91591-9d68-42ba-9f1b-1ef42f215e35', 'test', '/', 'test@example.com', '', 'YX6V08gSdP+RYgRtaVePad6wdVw=', 'question', 'dnc4cGDJy7KCmNn7vJJHG5OCbdU=', true, '2007-05-23 10:43:21.694593+02', '2007-05-23 10:43:21.773929+02', '2007-05-23 10:43:21.694593+02', '2007-05-23 10:43:21.694593+02', NULL, false, '2007-05-23 10:43:21.694593+02', 0, '2007-05-23 10:43:21.694593+02', 0, '2007-05-23 10:43:21.694593+02');

INSERT INTO usersinroles (username, rolename, applicationname) VALUES ('test', 'Administrators', '/');

INSERT INTO Categories (Name, Description) VALUES ('Uncategorized', 'Uncategorized');
